import java.awt.Color;
import java.awt.Font;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
   
import javax.print.attribute.AttributeSet;
import javax.print.attribute.standard.JobName;
import javax.swing.JFrame;
import javax.swing.UIManager;

import jpdf.attribute.*;   
import jpdf.gui.dialog.PrintDialogPreferences;
import jpdf.gui.preview2.PageLayout;
import jpdf.gui.preview2.PreviewPrintableDialog;
import jpdf.manage.PrintManager;
import jpdf.manage.PrintParameterHolder;


/**
 * Class PrintableStylepadApplication adds print capabilities to the Stylepad application.  
 * The Stylepad is one of the demos that come with the Java SDK.  Three items are added
 * to the File Menu: Page Setup, Print Preview, and Print.  
 *
 * <P>This class was developed to serve as an example of how to use the JPDF to add 
 * printing capabilities to an existing application.
 *
 * <P><HR>
 * This application must be compiled and run using SDK version 1.5.  Version 1.5 
 * is required because of the way in which the Stylepad application was compiled.
 * <HR>
 */
public class PrintableStylepadApplication 
{
  //-----------------------------------------------------------------------
  // Inner Classes.
  //-----------------------------------------------------------------------

  /**
   * App Closer -- exits the application when the frame is closed.
   */
  protected static final class AppCloser 
    extends WindowAdapter 
  {
    public void windowClosing(WindowEvent theEvent) 
    {
      // Shutdown the JPDF Environment when the application exits.
      PrintManager.shutdownEnvironment();
  
      System.exit(0);
    }
  }


  //-----------------------------------------------------------------------
  // Public methods.
  //-----------------------------------------------------------------------

  /**
   * Application entry point.
   */
  public static void main(String[] args) 
  {
    // Initialize the JPDF Environment.
    initializeJpdfEnvironment();
    
    // Display the window.
    displayWindow();
  }
  
  
  //-----------------------------------------------------------------------
  // Private methods.
  //-----------------------------------------------------------------------

  /**
   * Displays the window containing the Printable Style Pad.
   */
  private static void displayWindow() 
  {
    PrintableStylepad theStylePad = new PrintableStylepad();
    JFrame            theWindow = new JFrame("Printable Stylepad");
   
    // Create and configure the frame.
    theWindow.getContentPane().add(theStylePad);
    theWindow.addWindowListener(new AppCloser());
    theWindow.setJMenuBar(theStylePad.createMenubar());
    theWindow.pack();
    theWindow.setSize(650, 700);
    theWindow.setLocationRelativeTo(null);
    
    // Display the frame.
    theWindow.setVisible(true);
  }   
  
  /**
   * Initializes the JPDF environment.
   */
  private static void initializeJpdfEnvironment() 
  {
    AttributeSet  theAttributeSet = null; 
    Font          thePageNoteFont = null;
    String        theDateAndTimeMarker = null;
    String        thePageAndCountMarker = null;

    // Initialize the JPDF Environment.
    try 
    {
      PrintManager.initializeEnvironment();
    }
    catch (Throwable theEx)
    {
      theEx.printStackTrace();
      System.exit(99); 
    }  
    
    // Get the Default Attribute Set.
    theAttributeSet = PrintManager.getDefaultPrintAttributes(); 

    // Add the initial attributes we want to use for printing.
    // NOTE: Special markers are used in the Page Header/Footer.  These are substituted by actual values
    //       when the material is printed.
    theDateAndTimeMarker = PrintParameterHolder.DATE_MARKER + " " + PrintParameterHolder.TIME_MARKER;
    thePageAndCountMarker = "Page " + PrintParameterHolder.PAGE_NUMBER_MARKER + " of " + 
                            PrintParameterHolder.PAGE_COUNT_MARKER;
    thePageNoteFont = UIManager.getFont("Label.font").deriveFont(Font.PLAIN, 12.0f);
    theAttributeSet.add(new JobName("Wonderland Example", null));
    theAttributeSet.add(new PageHeadNotes(PrintParameterHolder.TITLE_MARKER, "", theDateAndTimeMarker, 
                                          Color.BLUE, thePageNoteFont));
    theAttributeSet.add(new PageFootNotes("", thePageAndCountMarker, "", Color.BLUE, thePageNoteFont));
    theAttributeSet.add(PageFrame.TRUE);
    theAttributeSet.add(UseBackgroundColor.FALSE);
    theAttributeSet.add(BackgroundColorOnPage.TRUE);

    // Preload the list of media supported by the Default Print Service on a background thread.  
    // This speeds up the initial display of the Page Setup dialog.
    PrintManager.preloadPrintServiceMediaOnThread();   

    // Set the JPDF preferences.
    PrintDialogPreferences.setToUsePrintServiceSelectionIcons(true);
    
    // Set the Preview Dialog preferences.
    PreviewPrintableDialog.setInitialPageLayout(PageLayout.FULL_GRID);
    PreviewPrintableDialog.setInitialZoomFitPage();    
    PreviewPrintableDialog.setToUsePreviousSizeLocation(true);
  }
}